{
  "cells": [
    {
      "cell_type": "markdown",
      "metadata": {},
      "source": [
        "# EMIT: Classify interference type GUI\r\n\r\nThis example uses a GUI to open an AEDT project with an EMIT design and\r\nanalyze the results to classify the worst-case interference.\r\n"
      ]
    },
    {
      "cell_type": "markdown",
      "metadata": {},
      "source": [
        "# Perform required imports\r\n\r\nPerform required imports.\r\n"
      ]
    },
    {
      "cell_type": "code",
      "execution_count": null,
      "metadata": {
        "collapsed": false
      },
      "outputs": [],
      "source": [
        "import sys\nfrom pyaedt.emit_core.emit_constants import InterfererType, ResultType, TxRxMode\nfrom pyaedt import Emit\nfrom pyaedt import get_pyaedt_app\nimport pyaedt\nimport os\nimport subprocess\nimport pyaedt.generic.constants as consts\n\n# Check that emit is a compatible version\naedt_version = \"2024.1\"\nif aedt_version < \"2023.2\":\n    print(\"Must have v2023.2 or later\")\n    sys.exit()\n\n# Check to see which Python libraries have been installed\nreqs = subprocess.check_output([sys.executable, '-m', 'pip', 'freeze'])\ninstalled_packages = [r.decode().split('==')[0] for r in reqs.split()]\n\n# Install required packages if they are not installed\ndef install(package):\n    subprocess.check_call([sys.executable, '-m', 'pip', 'install', package])\n\n# Install required libraries for GUI and Excel exporting (internet connection needed)\nrequired_packages = ['PySide6', 'openpyxl']\nfor package in required_packages:\n    if package not in installed_packages:\n        install(package)\n\n# Import PySide6 and openpyxl libraries\nfrom PySide6 import QtWidgets, QtUiTools, QtGui, QtCore\nfrom openpyxl.styles import PatternFill\nimport openpyxl\n\n# Uncomment if there are Qt plugin errors\n# import PySide6\n# dirname = os.path.dirname(PySide6.__file__)\n# plugin_path = os.path.join(dirname, 'plugins', 'platforms')\n# os.environ['QT_QPA_PLATFORM_PLUGIN_PATH'] = plugin_path\n\n# Launch EMIT\nnon_graphical = False\nnew_thread = True\ndesktop = pyaedt.launch_desktop(aedt_version, non_graphical, new_thread)\n\n# Add emitapi to system path\nemit_path = os.path.join(desktop.install_path, \"Delcross\")\nsys.path.insert(0,emit_path)\nimport EmitApiPython\napi = EmitApiPython.EmitApi()\n\n# Define .ui file for GUI\nui_file = pyaedt.downloads.download_file(\"emit\", \"interference_gui.ui\")\nUi_MainWindow, _ = QtUiTools.loadUiType(ui_file)\n\nclass DoubleDelegate(QtWidgets.QStyledItemDelegate):\n    def __init__(self, decimals, values, max_power, min_power):\n        super().__init__()\n        self.decimals = decimals\n        self.values = values\n        self.max_power = max_power\n        self.min_power = min_power\n\n    def createEditor(self, parent, option, index):\n        editor = super().createEditor(parent, option, index)\n        if isinstance(editor, QtWidgets.QLineEdit):\n            validator = QtGui.QDoubleValidator(parent)\n            num_rows = len(self.values)\n            cur_row = index.row()\n            if cur_row == 0:\n                min_val = self.values[1]\n                max_val = self.max_power\n            elif cur_row == num_rows - 1:\n                min_val = self.min_power\n                max_val = self.values[cur_row-1]\n            else:\n                min_val = self.values[cur_row + 1]\n                max_val = self.values[cur_row - 1]\n            validator.setRange(min_val, max_val, self.decimals)\n            validator.setNotation(QtGui.QDoubleValidator.Notation.StandardNotation)\n            editor.setValidator(validator)\n        return editor\n\n    def update_values(self, values):\n        self.values = values\n\nclass MainWindow(QtWidgets.QMainWindow, Ui_MainWindow):\n    def __init__(self):\n        super(MainWindow, self).__init__()\n        self.emitapp = None\n        self.populating_dropdown = False\n        self.setupUi(self)\n        self.setup_widgets()\n\n    ###############################################################################\n    # Setup widgets\n    # ~~~~~~~~~~~~~\n    # Define all widgets from the UI file, connect the widgets to functions, define \n    # table colors, and format table settings. \n    \n    def setup_widgets(self):\n        # Widget definitions for file selection/tab management\n        self.file_select_btn = self.findChild(QtWidgets.QToolButton, \"file_select_btn\")\n        self.file_path_box = self.findChild(QtWidgets.QLineEdit, \"file_path_box\") \n        self.design_name_dropdown = self.findChild(QtWidgets.QComboBox, \"design_name_dropdown\")\n        self.design_name_dropdown.setEnabled(True)\n        self.tab_widget = self.findChild(QtWidgets.QTabWidget, \"tab_widget\")\n\n        # Widget definitions for protection level classification\n        self.protection_results_btn = self.findChild(QtWidgets.QPushButton, \"protection_results_btn\")\n        self.protection_matrix = self.findChild(QtWidgets.QTableWidget, \"protection_matrix\")    \n        self.protection_legend_table = self.findChild(QtWidgets.QTableWidget, \"protection_legend_table\")\n\n        self.damage_check = self.findChild(QtWidgets.QCheckBox, \"damage_check\")\n        self.overload_check = self.findChild(QtWidgets.QCheckBox, \"overload_check\")\n        self.intermodulation_check = self.findChild(QtWidgets.QCheckBox, \"intermodulation_check\")\n        self.desensitization_check = self.findChild(QtWidgets.QCheckBox, \"desensitization_check\")\n        self.protection_export_btn = self.findChild(QtWidgets.QPushButton, \"protection_export_btn\")\n        self.radio_specific_levels = self.findChild(QtWidgets.QCheckBox, \"radio_specific_levels\")\n        self.radio_dropdown = self.findChild(QtWidgets.QComboBox, \"radio_dropdown\")\n        self.protection_save_img_btn = self.findChild(QtWidgets.QPushButton, 'protection_save_img_btn')\n\n        # warning label\n        self.warning_label = self.findChild(QtWidgets.QLabel, \"warnings\")\n        myFont = QtGui.QFont()\n        myFont.setBold(True)\n        self.warning_label.setFont(myFont)\n        self.warning_label.setHidden(True)\n        self.design_name_dropdown.currentIndexChanged.connect(self.design_dropdown_changed)\n\n        # Setup for protection level buttons and table\n        self.protection_results_btn.setEnabled(False)\n        self.protection_export_btn.setEnabled(False)\n        self.protection_save_img_btn.setEnabled(False)\n        self.file_select_btn.clicked.connect(self.open_file_dialog)\n        self.protection_export_btn.clicked.connect(self.save_results_excel)\n        self.protection_results_btn.clicked.connect(self.protection_results)\n        self.protection_legend_table.resizeRowsToContents()\n        self.protection_legend_table.resizeColumnsToContents()\n        self.damage_check.stateChanged.connect(self.protection_results)\n        self.overload_check.stateChanged.connect(self.protection_results)\n        self.intermodulation_check.stateChanged.connect(self.protection_results)\n        self.desensitization_check.stateChanged.connect(self.protection_results)\n        self.protection_legend_table.setEditTriggers(QtWidgets.QTableWidget.DoubleClicked)\n        self.global_protection_level = True\n        self.protection_levels = {}\n        values = [float(self.protection_legend_table.item(row, 0).text()) for row in range(self.protection_legend_table.rowCount())]\n        self.protection_levels['Global'] = values\n        self.changing = False\n        self.radio_dropdown.currentIndexChanged.connect(self.radio_dropdown_changed)\n        self.protection_legend_table.itemChanged.connect(self.table_changed)\n        self.protection_save_img_btn.clicked.connect(self.save_image)\n\n        # Widget definitions for interference type\n        self.interference_results_btn = self.findChild(QtWidgets.QPushButton, \"interference_results_btn\")\n        self.interference_matrix = self.findChild(QtWidgets.QTableWidget, \"interference_matrix\")    \n        self.interference_legend_table = self.findChild(QtWidgets.QTableWidget, \"interference_legend_table\")\n\n        # set the items read only\n        for i in range(0, self.interference_legend_table.rowCount()):\n            item = self.interference_legend_table.item(i, 0)\n            item.setFlags(QtCore.Qt.ItemIsEnabled)\n            self.interference_legend_table.setItem(i, 0, item)\n\n        self.in_in_check = self.findChild(QtWidgets.QCheckBox, \"in_in_check\")\n        self.in_out_check = self.findChild(QtWidgets.QCheckBox, \"in_out_check\")\n        self.out_in_check = self.findChild(QtWidgets.QCheckBox, \"out_in_check\")\n        self.out_out_check = self.findChild(QtWidgets.QCheckBox, \"out_out_check\")\n        self.interference_export_btn = self.findChild(QtWidgets.QPushButton, \"interference_export_btn\")\n        self.interference_save_img_btn = self.findChild(QtWidgets.QPushButton, 'interference_save_img_btn')\n        \n        # Setup for interference type buttons and table\n        self.interference_results_btn.setEnabled(False)\n        self.interference_export_btn.setEnabled(False)\n        self.interference_save_img_btn.setEnabled(False)\n        self.interference_export_btn.clicked.connect(self.save_results_excel)\n        self.interference_results_btn.clicked.connect(self.interference_results)\n        self.interference_legend_table.resizeRowsToContents() \n        self.interference_legend_table.resizeColumnsToContents()\n        self.in_in_check.stateChanged.connect(self.interference_results)\n        self.in_out_check.stateChanged.connect(self.interference_results)\n        self.out_in_check.stateChanged.connect(self.interference_results)\n        self.out_out_check.stateChanged.connect(self.interference_results)\n        self.radio_specific_levels.stateChanged.connect(self.radio_specific)\n        self.interference_save_img_btn.clicked.connect(self.save_image)\n\n        # Color definition dictionary and previous project/design names\n        self.color_dict = {\"green\": [QtGui.QColor(125, 115, 202),'#7d73ca'], \n                           \"yellow\":[QtGui.QColor(211, 89, 162), '#d359a2'], \n                           \"orange\": [QtGui.QColor(255, 99, 97), '#ff6361'], \n                           \"red\": [QtGui.QColor(255, 166, 0), '#ffa600'], \n                           \"white\": [QtGui.QColor(\"white\"),'#ffffff']}\n        self.previous_design = ''\n        self.previous_project = ''\n\n        # Set the legend tables to strech resize mode\n        header = self.protection_legend_table.horizontalHeader()\n        v_header = self.protection_legend_table.verticalHeader()\n\n        header.setSectionResizeMode(0, QtWidgets.QHeaderView.ResizeMode.Stretch)\n        v_header.setSectionResizeMode(0, QtWidgets.QHeaderView.ResizeMode.Stretch)\n        v_header.setSectionResizeMode(1, QtWidgets.QHeaderView.ResizeMode.Stretch)\n        v_header.setSectionResizeMode(2, QtWidgets.QHeaderView.ResizeMode.Stretch)\n        v_header.setSectionResizeMode(3, QtWidgets.QHeaderView.ResizeMode.Stretch)\n\n        header = self.interference_legend_table.horizontalHeader()\n        v_header = self.interference_legend_table.verticalHeader()\n\n        header.setSectionResizeMode(0, QtWidgets.QHeaderView.ResizeMode.Stretch)\n        v_header.setSectionResizeMode(0, QtWidgets.QHeaderView.ResizeMode.Stretch)\n        v_header.setSectionResizeMode(1, QtWidgets.QHeaderView.ResizeMode.Stretch)\n        v_header.setSectionResizeMode(2, QtWidgets.QHeaderView.ResizeMode.Stretch)\n        v_header.setSectionResizeMode(3, QtWidgets.QHeaderView.ResizeMode.Stretch)\n\n        # Input validation for protection level legend table\n        self.delegate = DoubleDelegate(decimals=2, values=values,\n                                  max_power=1000, min_power=-200)\n        self.protection_legend_table.setItemDelegateForColumn(0, self.delegate)\n        self.open_file_dialog()\n\n    ###############################################################################\n    # Open file dialog and select project\n    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n    # Open the file dialog for project selection and populate the design dropdown \n    # with all EMIT designs in the project. \n\n    def open_file_dialog(self):\n        fname, _filter = QtWidgets.QFileDialog.getOpenFileName(self, \"Select EMIT Project\", \"\", \"Ansys Electronics Desktop Files (*.aedt)\", )\n        if fname: \n            self.file_path_box.setText(fname)\n\n            # Close previous project and open specified one\n            if self.emitapp is not None:\n                self.emitapp.close_project()\n                self.emitapp = None\n            desktop_proj = desktop.load_project(self.file_path_box.text())\n\n            # check for an empty project (i.e. no designs)\n            if isinstance(desktop_proj, bool):\n                self.file_path_box.setText(\"\")\n                msg = QtWidgets.QMessageBox()\n                msg.setWindowTitle(\"Error: Project missing designs.\")\n                msg.setText(\n                    \"The selected project has no designs. Projects must have at least \"\n                    \"one EMIT design. See AEDT log for more information.\")\n                x = msg.exec()\n                return\n\n            # Check if project is already open\n            if desktop_proj.lock_file == None:\n                msg = QtWidgets.QMessageBox()\n                msg.setWindowTitle(\"Error: Project already open\")\n                msg.setText(\"Project is locked. Close or remove the lock before proceeding. See AEDT log for more information.\")\n                x = msg.exec()\n                return\n            \n            # Populate design dropdown with all design names\n            designs = desktop_proj.design_list\n            emit_designs = []\n            self.populating_dropdown = True\n            self.design_name_dropdown.clear()\n            self.populating_dropdown = False\n            for d in designs:\n                design_type = desktop.design_type(desktop_proj.project_name, d)\n                if design_type == \"EMIT\":\n                    emit_designs.append(d)\n\n            # add warning if no EMIT design\n            # Note: this should never happen since loading a project without an EMIT design\n            # should add a blank EMIT design\n            self.warning_label.setHidden(True)\n            if len(emit_designs) == 0:\n                self.warning_label.setText(\"Warning: The project must contain at least one EMIT design.\")\n                self.warning_label.setHidden(False)\n                return\n\n            self.populating_dropdown = True\n            self.design_name_dropdown.addItems(emit_designs)\n            self.populating_dropdown = False\n            self.emitapp = get_pyaedt_app(desktop_proj.project_name, emit_designs[0])\n            self.design_name_dropdown.setCurrentIndex(0)\n\n            # check for at least 2 radios\n            radios = self.emitapp.modeler.components.get_radios()\n            self.warning_label.setHidden(True)\n            if len(radios) < 2:\n                self.warning_label.setText(\"Warning: The selected design must contain at least two radios.\")\n                self.warning_label.setHidden(False)\n\n            if self.radio_specific_levels.isEnabled():\n                self.radio_specific_levels.setChecked(False)\n                self.radio_dropdown.clear()\n                self.radio_dropdown.setEnabled(False)\n                self.protection_levels = {}\n                values = [float(self.protection_legend_table.item(row, 0).text()) for row in range(self.protection_legend_table.rowCount())]\n                self.protection_levels['Global'] = values\n\n            self.radio_specific_levels.setEnabled(True)\n            self.protection_results_btn.setEnabled(True)\n            self.interference_results_btn.setEnabled(True)\n\n    ###############################################################################\n    # Change design selection\n    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n    # Refresh the warning messages when the selected design changes\n\n    def design_dropdown_changed(self):\n        if self.populating_dropdown:\n            # don't load design's on initial project load\n            return\n        design_name = self.design_name_dropdown.currentText()\n        self.emitapp = get_pyaedt_app(self.emitapp.project_name, design_name)\n        # check for at least 2 radios\n        radios = self.emitapp.modeler.components.get_radios()\n        self.warning_label.setHidden(True)\n        if len(radios) < 2:\n            self.warning_label.setText(\"Warning: The selected design must contain at least two radios.\")\n            self.warning_label.setHidden(False)\n\n        # clear the table if the design is changed\n        self.clear_table()\n\n    ###############################################################################\n    # Enable radio specific protection levels\n    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n    # Activate radio selection dropdown and initialize dictionary to store protection levels\n    # when the radio-specific level dropdown is checked.\n\n    def radio_specific(self):\n        self.radio_dropdown.setEnabled(self.radio_specific_levels.isChecked())\n        self.radio_dropdown.clear()\n        if self.radio_dropdown.isEnabled():\n            self.emitapp.set_active_design(self.design_name_dropdown.currentText())\n            radios = self.emitapp.modeler.components.get_radios()\n            values = [float(self.protection_legend_table.item(row, 0).text()) for row in range(self.protection_legend_table.rowCount())]\n            for radio in radios:\n                if radios[radio].has_rx_channels():\n                    self.protection_levels[radio] = values\n                    self.radio_dropdown.addItem(radio)\n        else:\n            self.radio_dropdown.clear()\n            values = [float(self.protection_legend_table.item(row, 0).text()) for row in range(self.protection_legend_table.rowCount())]\n            self.protection_levels['Global'] = values\n        self.global_protection_level = not self.radio_specific_levels.isChecked()\n    \n    ###############################################################################\n    # Update legend table\n    # ~~~~~~~~~~~~~~~~~~~\n    # Update shown legend table values when the radio dropdown value changes. \n\n    def radio_dropdown_changed(self):\n        if self.radio_dropdown.isEnabled():\n            self.changing = True\n            for row in range(self.protection_legend_table.rowCount()):\n                item = self.protection_legend_table.item(row, 0)\n                item.setText(str(self.protection_levels[self.radio_dropdown.currentText()][row]))\n            self.changing = False\n            # update the validator so min/max for each row is properly set\n            values = [float(self.protection_legend_table.item(row, 0).text()) for row in\n                      range(self.protection_legend_table.rowCount())]\n            self.delegate.update_values(values)\n\n    ###############################################################################\n    # Save legend table values\n    # ~~~~~~~~~~~~~~~~~~~~~~~~\n    # Save inputted radio protection level threshold values every time one is changed \n    # in the legend table.\n\n    def table_changed(self):\n        if self.changing == False:\n            values = [float(self.protection_legend_table.item(row, 0).text()) for row in\n                      range(self.protection_legend_table.rowCount())]\n            if self.radio_dropdown.currentText() == '':\n                index = 'Global'\n            else:\n                index = self.radio_dropdown.currentText()\n            self.protection_levels[index] = values\n            self.delegate.update_values(values)\n\n    ###############################################################################\n    # Save scenario matrix to as PNG file \n    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n    # Save the scenario matrix table as a PNG file.\n\n    def save_image(self):\n        if self.tab_widget.currentIndex() == 0:\n            table = self.protection_matrix\n        else:\n            table = self.interference_matrix\n\n        fname, _filter = QtWidgets.QFileDialog.getSaveFileName(self, \"Save Scenario Matrix\", \"Scenario Matrix\", \"png (*.png)\")\n        if fname:\n            image = QtGui.QImage(table.size(), QtGui.QImage.Format_ARGB32)\n            table.render(image)\n            image.save(fname)\n\n    ###############################################################################\n    # Save scenario matrix to Excel file \n    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n    # Write the scenario matrix results to an Excel file with color coding.\n\n    def save_results_excel(self):\n        defaultName = \"\"\n        if self.tab_widget.currentIndex() == 0:\n            table = self.protection_matrix\n            defaultName = \"Protection Level Classification\"\n        else:\n            table = self.interference_matrix\n            defaultName = \"Interference Type Classification\"\n\n        fname, _filter = QtWidgets.QFileDialog.getSaveFileName(self, \"Save Scenario Matrix\", defaultName, \"xlsx (*.xlsx)\")\n\n        if fname:\n            workbook = openpyxl.Workbook()\n            worksheet = workbook.active\n            header = self.tx_radios[:]\n            header.insert(0, \"Tx/Rx\")\n            worksheet.append(header)\n            for row in range(2, table.rowCount()+2):\n                worksheet.cell(row = row, column = 1, value = str(self.rx_radios[row-2]))\n                for col in range(2, table.columnCount()+2):\n                    text = str(table.item(row-2, col-2).text())\n                    worksheet.cell(row = row, column = col, value = text)\n                    cell = worksheet.cell(row, col)\n                    cell.fill = PatternFill(start_color = self.color_dict[self.all_colors[col-2][row-2]][1][1:], \n                                            end_color = self.color_dict[self.all_colors[col-2][row-2]][1][1:], \n                                            fill_type = \"solid\")\n            workbook.save(fname)\n\n    ###############################################################################\n    # Run interference type simulation \n    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n    # Run interference type simulation and classify results.\n\n    def interference_results(self):\n        # Initialize filter check marks and expected filter results\n        self.interference_checks = [self.in_in_check.isChecked(), self.out_in_check.isChecked(),\n                                  self.in_out_check.isChecked(), self.out_out_check.isChecked()]\n\n        self.interference_filters =[\"TxFundamental:In-band\", [\"TxHarmonic/Spurious:In-band\",\"Intermod:In-band\", \"Broadband:In-band\"], \n                                    \"TxFundamental:Out-of-band\", [\"TxHarmonic/Spurious:Out-of-band\",\"Intermod:Out-of-band\", \"Broadband:Out-of-band\"]]\n\n        # Create list of problem types to analyze according to inputted filters\n        filter = [i for (i,v) in zip(self.interference_filters, self.interference_checks) if v]\n\n        if self.file_path_box.text() != \"\" and self.design_name_dropdown.currentText() != \"\":\n            if self.previous_design != self.design_name_dropdown.currentText() or self.previous_project != self.file_path_box.text():\n                self.previous_design = self.design_name_dropdown.currentText()\n                self.previous_project = self.file_path_box.text()\n                self.emitapp.set_active_design(self.design_name_dropdown.currentText())\n\n                # Check if file is read-only\n                if self.emitapp.save_project() == False:\n                    msg = QtWidgets.QMessageBox()\n                    msg.setWindowTitle(\"Writing Error\")\n                    msg.setText(\"An error occurred while writing to the file. Is it readonly? Disk full? See AEDT log for more information.\")\n                    x = msg.exec()\n                    return\n\n                # Get results and radios\n                self.rev = self.emitapp.results.analyze()\n                self.tx_interferer = InterfererType().TRANSMITTERS\n                self.rx_radios = self.rev.get_receiver_names()\n                self.tx_radios = self.rev.get_interferer_names(self.tx_interferer)\n\n                # Check if design is valid\n                if self.tx_radios is None or self.rx_radios is None:\n                    return\n\n            # Classify the interference\n            # ~~~~~~~~~~~~~~~~~~~~~~~~~~~\n            # Iterate over all the transmitters and receivers and compute the power\n            # at the input to each receiver due to each of the transmitters. Compute\n            # which, if any, type of interference occurred.\n            domain = self.emitapp.results.interaction_domain()\n            self.all_colors, self.power_matrix = self.rev.interference_type_classification(domain, use_filter = True, filter_list = filter)\n\n            # Save project and plot results on table widget\n            self.emitapp.save_project()\n            self.populate_table()\n\n    ###############################################################################\n    # Run protection level simulation \n    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n    # Run protection level simulation and classify results accroding to inputted \n    # threshold levels.\n\n    def protection_results(self):\n        # Initialize filter check marks and expected filter results\n        self.protection_checks = [self.damage_check.isChecked(), self.overload_check.isChecked(),\n                                  self.intermodulation_check.isChecked(), self.desensitization_check.isChecked()]\n\n        self.protection_filters = ['damage', 'overload', 'intermodulation', 'desensitization']\n\n        filter = [i for (i,v) in zip(self.protection_filters, self.protection_checks) if v]\n\n        if self.file_path_box.text() != \"\" and self.design_name_dropdown.currentText() != \"\":\n            if self.previous_design != self.design_name_dropdown.currentText() or self.previous_project != self.file_path_box.text():\n                self.previous_design = self.design_name_dropdown.currentText()\n                self.previous_project = self.file_path_box.text()\n                self.emitapp.set_active_design(self.design_name_dropdown.currentText())\n\n                # Check if file is read-only\n                if self.emitapp.save_project() == False:\n                    msg = QtWidgets.QMessageBox()\n                    msg.setWindowTitle(\"Writing Error\")\n                    msg.setText(\"An error occurred while writing to the file. Is it readonly? Disk full? See AEDT log for more information.\")\n                    x = msg.exec()\n                    return\n                \n                # Get results and design radios\n                self.tx_interferer = InterfererType().TRANSMITTERS\n                self.rev = self.emitapp.results.analyze()\n                self.rx_radios = self.rev.get_receiver_names()\n                self.tx_radios = self.rev.get_interferer_names(self.tx_interferer)\n\n            # Check if there are radios in the design\n            if self.tx_radios is None or self.rx_radios is None:\n                return\n\n            domain = self.emitapp.results.interaction_domain()\n            self.all_colors, self.power_matrix = self.rev.protection_level_classification(domain, \n                                                                                 self.global_protection_level, \n                                                                                 self.protection_levels['Global'], \n                                                                                 self.protection_levels, use_filter = True, \n                                                                                 filter_list = filter)\n\n            self.populate_table()\n    \n    ###############################################################################\n    # Populate the scenario matrix\n    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n    # Create a scenario matrix view with the transmitters defined across the top\n    # and receivers down the left-most column.\n\n    def populate_table(self):\n        if self.tab_widget.currentIndex() == 0:\n            table = self.protection_matrix\n            button = self.protection_export_btn\n            img_btn = self.protection_save_img_btn\n        else:\n            table = self.interference_matrix\n            button = self.interference_export_btn\n            img_btn = self.interference_save_img_btn\n\n        num_cols = len(self.all_colors)\n        num_rows = len(self.all_colors[0])\n        table.setColumnCount(num_cols)\n        table.setRowCount(num_rows)\n        table.setVerticalHeaderLabels(self.rx_radios)\n        table.setHorizontalHeaderLabels(self.tx_radios)\n\n        for col in range(num_cols):\n            for row in range(num_rows):\n                item = QtWidgets.QTableWidgetItem(str(self.power_matrix[col][row]))\n                table.setItem(row, col, item)\n                cell = table.item(row, col)\n                cell.setBackground(self.color_dict[self.all_colors[col][row]][0])\n        \n        button.setEnabled(True)\n        img_btn.setEnabled(True)\n\n    def clear_table(self):\n        # get the table/buttons based on current tab\n        if self.tab_widget.currentIndex() == 0:\n            table = self.protection_matrix\n            button = self.protection_export_btn\n            img_btn = self.protection_save_img_btn\n        else:\n            table = self.interference_matrix\n            button = self.interference_export_btn\n            img_btn = self.interference_save_img_btn\n\n        # disable export options\n        button.setEnabled(False)\n        img_btn.setEnabled(False)\n\n        # clear the table\n        table.setColumnCount(0)\n        table.setRowCount(0)\n\n    ###############################################################################\n    # GUI closing event\n    # ~~~~~~~~~~~~~~~~~\n    # Close AEDT if the GUI is closed.\n    def closeEvent(self, event):\n        msg = QtWidgets.QMessageBox()\n        msg.setWindowTitle(\"Closing GUI\")\n        msg.setText(\"Closing AEDT. Wait for the GUI to close on its own.\")\n        x = msg.exec()\n        if self.emitapp:\n            self.emitapp.close_project()\n            self.emitapp.close_desktop()\n        else:\n            desktop.release_desktop(True, True)"
      ]
    },
    {
      "cell_type": "markdown",
      "metadata": {},
      "source": [
        "# Run GUI\r\n\r\nLaunch the GUI. If you want to run the GUI, uncomment the\r\n`window.show()` and `app.exec_()` method calls.\r\n"
      ]
    },
    {
      "cell_type": "code",
      "execution_count": null,
      "metadata": {
        "collapsed": false
      },
      "outputs": [],
      "source": [
        "if __name__ == '__main__' and os.getenv(\"PYAEDT_DOC_GENERATION\", \"False\") != \"1\":\n    app = QtWidgets.QApplication([])\n    window = MainWindow()\n    window.show()\n    app.exec()\nelse:\n    desktop.release_desktop(True, True)"
      ]
    }
  ],
  "metadata": {
    "kernelspec": {
      "display_name": "Python 3",
      "language": "python",
      "name": "python3"
    },
    "language_info": {
      "codemirror_mode": {
        "name": "ipython",
        "version": 3
      },
      "file_extension": ".py",
      "mimetype": "text/x-python",
      "name": "python",
      "nbconvert_exporter": "python",
      "pygments_lexer": "ipython3",
      "version": "3.10.9"
    }
  },
  "nbformat": 4,
  "nbformat_minor": 0
}